       subroutine lagrangian
c-------------------------------------------------------------------------       
c  This subroutine contains the total lagrangain from which the accelerations
c  are obtained.  For numerical gradient checking, the derivetive of the     
c  lagrangian in the velocities and coordinates are calculated seperately      
c-------------------------------------------------------------------------
       include 'DIMENSIONS'
       include 'COMMON.VAR'
       include 'COMMON.CHAIN'
       include 'COMMON.DERIV'
       include 'COMMON.GEO'
       include 'COMMON.LOCAL'
       include 'COMMON.INTERACT'
       include 'COMMON.MD'
       include 'COMMON.IOUNITS'
       
       integer i,j,ind
       double precision zapas(MAXRES6)
       logical lprn /.false./  

       do j=1,3
         zapas(j)=-gcart(j,0)
       enddo
      ind=3      
      if (lprn) then
        write (iout,*) "Potential forces backbone"
      endif
      do i=nnt,nct-1
        if (lprn) write (iout,'(i5,3e15.5,5x,3e15.5)') 
     &    i,(-gcart(j,i),j=1,3)
        do j=1,3
          ind=ind+1
          zapas(ind)=-gcart(j,i)
        enddo
      enddo
      if (lprn) write (iout,*) "Potential forces sidechain"
      do i=nnt,nct
        if (itype(i).ne.10) then
          if (lprn) write (iout,'(i5,3e15.5,5x,3e15.5)') 
     &       i,(-gcart(j,i),j=1,3)
          do j=1,3
            ind=ind+1
            zapas(ind)=-gxcart(j,i)
          enddo
        endif
      enddo

      do i=1,dimen
        d_a_work(i)=0.0d0
        do j=1,dimen
          d_a_work(i)=d_a_work(i)+Ginv(i,j)*zapas(j)
        enddo
      enddo 
       
      do j=1,3
        d_a(j,0)=d_a_work(j)
      enddo
      ind=3
      do i=nnt,nct-1
        do j=1,3
          ind=ind+1
          d_a(j,i)=d_a_work(ind)
        enddo
      enddo
      do i=nnt,nct
        if (itype(i).ne.10) then
          do j=1,3
            ind=ind+1
            d_a(j,i+nres)=d_a_work(ind)
          enddo
        endif
      enddo
      if (lprn) then
        write(iout,*) 'acceleration 3D'
        write (iout,'(i3,3f10.5,3x,3f10.5)') 0,(d_a(j,0),j=1,3)
        do i=nnt,nct-1
         write (iout,'(i3,3f10.5,3x,3f10.5)') i,(d_a(j,i),j=1,3)
        enddo
        do i=nnt,nct
         write (iout,'(i3,3f10.5,3x,3f10.5)') 
     &     i+nres,(d_a(j,i+nres),j=1,3)
        enddo
      endif
      return        
      end             			    	      	  	  
c------------------------------------------------------------------
      subroutine setup_MD_matrices
      include 'DIMENSIONS'
      include 'COMMON.VAR'
      include 'COMMON.CHAIN'
      include 'COMMON.DERIV'
      include 'COMMON.GEO'
      include 'COMMON.LOCAL'
      include 'COMMON.INTERACT'
      include 'COMMON.MD'
      include 'COMMON.LANGEVIN'
      include 'COMMON.IOUNITS'
      integer i,j
      logical lprn /.true./
      logical sing
      double precision dtdi,massvec(maxres6),Gcopy(maxres6,maxres6),
     &  Ghalf(mmaxres6),sqreig(maxres6)
      double precision work(8*maxres6)
      integer iwork(maxres6)
c
c Set up the matrix of the (dC,dX)-->(C,X) transformation (A), the
c inertia matrix (Gmat) and the inverse of the inertia matrix (Ginv)
c
c Determine the number of degrees of freedom (dimen) and the number of 
c sites (dimen1)
      dimen=(nct-nnt+1)*3+nside*3
      dimen1=(nct-nnt)*3+(nct-nnt+1)*3
      write (iout,*) "dimen",dimen," dimen1",dimen1
c  Zeroing out A and fricmat
      do i=1,dimen
        do j=1,dimen
          A(i,j)=0.0D0     
        enddo	 
      enddo
c  Diagonal elements of the dC part of A and the respective friction coefficients
      ind=1
      ind1=0
      do i=nnt,nct-1
        ind=ind+1
        ind1=ind1+1
        ii=3*(ind-1)
        ii1=3*(ind1-1)
        coeff=0.25d0*IP
        massvec(ii1+1)=mp
        massvec(ii1+2)=mp
        massvec(ii1+3)=mp
        Gmat(ii+1,ii+1)=coeff
        Gmat(ii+2,ii+2)=coeff
        Gmat(ii+3,ii+3)=coeff
        A(ii1+1,ii+1)=0.5d0
        A(ii1+2,ii+2)=0.5d0
        A(ii1+3,ii+3)=0.5d0
      enddo
      
c  Off-diagonal elements of the dC part of A 
      k=3
      do i=1,nct-nnt
        do j=1,i
          do l=1,3
            A(3*(i-1)+l,3*(j-1)+l)=1.0d0
          enddo
        enddo
      enddo
c  Diagonal elements of the dX part of A and the respective friction coefficients
      m=nct-nnt
      m1=nct-nnt+1
      ind=0
      ind1=0
      do i=nnt,nct
        ind=ind+1
        ii = 3*(ind-1+m)
        iti=itype(i)
        massvec(ii+1)=msc(iti)
        massvec(ii+2)=msc(iti)
        massvec(ii+3)=msc(iti)
        if (iti.ne.10) then
          ind1=ind1+1
          ii1= 3*(ind1-1+m1)
          A(ii+1,ii1+1)=1.0d0
          A(ii+2,ii1+2)=1.0d0
          A(ii+3,ii1+3)=1.0d0
          Gmat(ii1+1,ii1+1)=ISC(iti)
          Gmat(ii1+2,ii1+2)=ISC(iti)
          Gmat(ii1+3,ii1+3)=ISC(iti)
        endif
      enddo
c  Off-diagonal elements of the dX part of A
      ind=0
      k=3*(nct-nnt)
      do i=nnt,nct
        iti=itype(i)
        ind=ind+1
        do j=nnt,i
          ii = 3*(ind-1)
          jj = 3*(j-nnt)
          do l=1,3
            A(k+ii+l,jj+l)=1.0d0
          enddo
        enddo
      enddo
c Calculate the G matrix (store in Gmat)
      do k=1,dimen
       do i=1,dimen
         dtdi=0.0d0
         do j=1,dimen1
           dtdi=dtdi+A(j,k)*A(j,i)*massvec(j)
         enddo
         Gmat(k,i)=Gmat(k,i)+dtdi
       enddo
      enddo 
      
      if (lprn) then
        write (iout,'(//a)') "Gmat"
        call matout(dimen,dimen,maxres6,maxres6,Gmat)
      endif
      do i=1,dimen
        do j=1,dimen
          Ginv(i,j)=0.0d0
          Gcopy(i,j)=Gmat(i,j)
        enddo
        Ginv(i,i)=1.0d0
      enddo
c Invert the G matrix
      call MATINVERT(dimen,maxres6,Gcopy,Ginv,sing)
      if (lprn) then
        write (iout,'(//a)') "Ginv"
        call matout(dimen,dimen,maxres6,maxres6,Ginv)
      endif
      if (sing) then
        write (iout,*) "The G matrix is singular."
        stop
      endif
c Compute G**(-1/2) and G**(1/2) 
      ind=0
      do i=1,dimen
        do j=1,i
          ind=ind+1
          Ghalf(ind)=Gmat(i,j)
        enddo
      enddo
      call gldiag(maxres6,dimen,dimen,Ghalf,work,Geigen,Gvec,
     &  ierr,iwork)
      if (lprn) then
        write (iout,'(//a)') 
     &   "Eigenvectors and eigenvalues of the G matrix"
        call eigout(dimen,dimen,maxres6,maxres6,Gvec,Geigen)
      endif
      do i=1,dimen
        sqreig(i)=dsqrt(Geigen(i))
      enddo
      do i=1,dimen
        do j=1,dimen
          Gsqrp(i,j)=0.0d0
          Gsqrm(i,j)=0.0d0
          Gcopy(i,j)=0.0d0
          do k=1,dimen
            Gsqrp(i,j)=Gsqrp(i,j)+Gvec(i,k)*Gvec(j,k)*sqreig(k)
            Gsqrm(i,j)=Gsqrm(i,j)+Gvec(i,k)*Gvec(j,k)/sqreig(k)
            Gcopy(i,j)=Gcopy(i,j)+Gvec(i,k)*Gvec(j,k)*Geigen(k)
          enddo
        enddo
      enddo
c      if (lprn) then
c        write (iout,*) "Comparison of original and restored G"
c        do i=1,dimen
c          do j=1,dimen
c            write (iout,'(2i5,5f10.5)') i,j,Gmat(i,j),Gcopy(i,j),
c     &        Gmat(i,j)-Gcopy(i,j),Gsqrp(i,j),Gsqrm(i,j)
c          enddo
c        enddo
c      endif
      return
      end 
c----------------------------------------------------------------------------------------
      SUBROUTINE EIGOUT(NC,NR,LM2,LM3,A,B)
      include 'DIMENSIONS'
      include 'COMMON.IOUNITS'
      double precision A(LM2,LM3),B(LM2)
      KA=1
      KC=6
    1 KB=MIN0(KC,NC)
      WRITE(IOUT,600) (I,I=KA,KB)
      WRITE(IOUT,601) (B(I),I=KA,KB)
      WRITE(IOUT,602)
    2 N=0
      DO 3  I=1,NR
      WRITE(IOUT,603) I,(A(I,J),J=KA,KB)
      N=N+1
      IF(N.LT.10) GO TO 3
      WRITE(IOUT,602)
      N=0
    3 CONTINUE
    4 IF (KB.EQ.NC) RETURN
      KA=KC+1
      KC=KC+6
      GO TO 1
  600 FORMAT (// 9H ROOT NO.,I4,9I11)
  601 FORMAT (/5X,10(1PE11.4))
  602 FORMAT (2H  )
  603 FORMAT (I5,10F11.5)
  604 FORMAT (1H1)
      END
c----------------------------------------------------------------------------------------
      SUBROUTINE MATOUT(NC,NR,LM2,LM3,A)
      include 'DIMENSIONS'
      include 'COMMON.IOUNITS'
      double precision A(LM2,LM3)
      KA=1
      KC=6
    1 KB=MIN0(KC,NC)
      WRITE(IOUT,600) (I,I=KA,KB)
      WRITE(IOUT,602)
    2 N=0
      DO 3  I=1,NR
      WRITE(IOUT,603) I,(A(I,J),J=KA,KB)
      N=N+1
      IF(N.LT.10) GO TO 3
      WRITE(IOUT,602)
      N=0
    3 CONTINUE
    4 IF (KB.EQ.NC) RETURN
      KA=KC+1
      KC=KC+6
      GO TO 1
  600 FORMAT (//5x,9I11)
  602 FORMAT (2H  )
  603 FORMAT (I5,10F11.3)
  604 FORMAT (1H1)
      END
